REM CRYPTO28 - SUPER ENCIPHERMENT TABLES                          (2 AUG 95)
REM $DYNAMIC

'              USES RANDOM NUMBERS TO CREATE SUPERENCIPHERMENT TABLES FOR USE
'              IN THE ENIGMA95 CIPHER MACHINE (ENIGMA95.BAS).

'              THESE TABLES ARE USED TO CONVERT NUMERIC CODE OUTPUT INTO
'              ALPHA CODE OUTPUT AND VICE VERSA.

'              THIS PROGRAM PRODUCES:

'              1 - A Super Enciphering Table (100 x 8) in which each digit
'                  from 1 to 100 can be represented by 6 or 7 2-letter
'                  combinations from AA to ZZ.  (e.g., 21 = LZ)

'              2 - A Super Deciphering Table (26 x 26) which allows you to
'                  determine what any given 2-letter combination translates
'                  to numerically. (e.g., LZ = 21)

'              These tables are first printed on paper for a hard copy, then
'              can be stored on floppy disk for use by ENIGMA95.BAS.

'              Any Super Encipherment Table can be recalled from disk for
'              inspection.

'--------------------------------------------------------------------------
START:
CLEAR

  OPTION BASE 1

  DIM C(100)           ' COUNTER ARRAY FOR ALPHA CHARACTER CELLS
  DIM F$(100, 8)       ' FINAL SUPERENCIPHERING ALPHA CODE ARRAY
  DIM J$(26, 26)       ' DECIPHERING ARRAY
  DIM R1$(1, 676)      ' FIRST TRANSFER ARRAY FOR DATA FROM Z$(26,26)
  DIM R2$(1, 676)      ' FINAL DATA ARRAY
  DIM Z$(26, 26)       ' MATRIX FOR ALPHA CHARACTERS
  
  H$ = STRING$(1600, "  ")            'DISK FILE

  C$ = "ABCDEFGHIJKLMNOPQRSTUVWXYZ"   'SOURCE FOR ALPHA KEYS

  GOSUB TAB13                         'CREATE RANDOM ALPHA KEYS

  LPRINT CHR$(27); "@";               'INITIALIZE PRINTER
  LPRINT CHR$(27); "x"; CHR$(1);      'LQ MODE
  LPRINT CHR$(27); "M";               'ELITE PRINT
  LPRINT CHR$(27); "k"; CHR$(1);      'SANS SERIF FONT

  WIDTH LPRINT 155                    'SET LINE WIDTH TO 155 COLUMNS
  LPRINT CHR$(27); "Q"; CHR$(162);    'SET RIGHT MARGIN TO 162 CHARACTERS"

'----------------------------------------------------------------------------

  CLS

  LOCATE 4, 12
  COLOR 14, 0                         'YELLOW/BLACK
  PRINT "WELCOME TO CRYPTO28 - SUPER ENCIPHERMENT TABLE GENERATOR"
  LOCATE 6, 30
  COLOR 27, 0
  PRINT "INSERT TABLES DISK"
 
  LOCATE 22, 30
  COLOR 12, 0                         'RED/BLACK
  PRINT "Clarence E. Tyner Jr."
  LOCATE 23, 15
  PRINT "Copyrighted 1995 - All Rights Reserved Worldwide."
  COLOR 7, 0                          'WHITE/BLACK
  F$ = ""
  LOCATE 10, 22
  PRINT "DO YOU WANT TO:"
  LOCATE 12, 20
  PRINT "  1 - CREATE A SUPER ENCIPHERMENT TABLE"
  LOCATE 14, 20
  PRINT "  2 - RETRIEVE A TABLE FROM FILE"
 
  DO WHILE F$ = ""
     F$ = INKEY$
  LOOP
  F = VAL(F$)
  IF F > 2 THEN GOTO START
  IF F < 1 THEN GOTO START
  IF F = 2 THEN GOTO TAB12

  CLS
  LOCATE 10, 10
  PRINT "ENTER TWO NUMBERS (1 to 26) TO SCRAMBLE ARRAY ALPHABETS"

REDO1:

  LOCATE 13, 20
  INPUT "ENTER THE 1ST NUMBER  :  ", A
  IF A < 1 OR A > 26 THEN
    BEEP
    GOTO REDO1
    END IF

REDO2:
  LOCATE 15, 20
  INPUT "ENTER THE 2ND NUMBER  :  ", B
  IF B < 1 OR B > 26 THEN
    BEEP
    GOTO REDO2
    END IF

  IF A = B THEN
    BEEP
    GOTO START
    END IF

  W = 0                               'FOR PRINTING F$(100 x 8)

  A1$ = MID$(A$, A)                   'TO REARRRANGE A$
  A2$ = MID$(A$, 1, A - 1)
  A$ = A1$ + A2$

  B1$ = MID$(B$, B)                   'TO REARRANGE B$
  B2$ = MID$(B$, 1, B - 1)
  B$ = B1$ + B2$

  FOR X = 1 TO 100: C(X) = 1: NEXT X  'INITIALIZE TABLE COUNTERS
 
  CLS

  PRINT "26 X 26 ARRAY BEING INITIALIZED"

  FOR X = 1 TO 26
    FOR Y = 1 TO 26
      Z$(X, Y) = "  "
    NEXT Y
  NEXT X

  PRINT "26 X 26 ARRAY BEING FILLED RANDOMLY"

  RANDOMIZE TIMER

  FOR X = 1 TO 26
    FOR Y = 1 TO 26

TAB1:
      R = RND
      Z1 = CINT(27 * R) - 1
      IF Z1 <= 0 THEN GOTO TAB1
      IF Z1 > 26 THEN GOTO TAB1

TAB2:
      C = RND
      Z2 = CINT(27 * C) - 1
      IF Z2 <= 0 THEN GOTO TAB2
      IF Z2 > 26 THEN GOTO TAB2

      IF Z$(Z1, Z2) <> "  " THEN GOTO TAB1
      MID$(Z$(Z1, Z2), 1, 1) = MID$(A$, X, 1)
      MID$(Z$(Z1, Z2), 2, 1) = MID$(B$, Y, 1)
      L = L + 1
 
      LOCATE 3, 1
      PRINT "CELL "; L; " OF 676 FILLED"

    NEXT Y
  NEXT X

  X = 0                               'PUT Z$(26,26) INTO R1$(1,676)
  FOR R = 1 TO 26
    FOR C = 1 TO 26
      X = X + 1
      IF X > 676 THEN EXIT FOR
      R1$(1, X) = Z$(R, C)
    NEXT C
  NEXT R
  X = 0

  PRINT "RANDOM 1 X 676 ARRAY BEING FORMED"

  RANDOMIZE TIMER
  C = 0
  H = 0

TAB4:
  X = RND                             'A RANDOM NUMBER BETWEEN 0 AND 1
  Z = CINT(677 * X) - 1               'PRODUCE A NUMBER BETWEEN 1 AND 676
  C = C + 1
  IF Z <= 0 THEN GOTO TAB4            'GET ANOTHER NUMBER
  IF Z > 676 GOTO TAB4                'GET ANOTHER NUMBER
  IF R1$(1, Z) = "  " THEN GOTO TAB4  'GET ANOTHER NUMBER

  H = H + 1
  R2$(1, H) = R1$(1, Z)               'MOVE DATA TO NEW (1X676) ARRAY RANDOMLY
  R1$(1, Z) = "  "
  IF H = 676 THEN GOTO TAB5
  GOTO TAB4

TAB5:
  PRINT
  PRINT "CREATING 100 X 8 SUPERENCIPHERMENT TABLE"

  V = 0
  FOR C = 1 TO 6                      'FILL COLUMNS 1 THRU 6
    FOR R = 1 TO 100                  'ROWS 1 TO 100
      V = V + 1
      F$(R, C) = R2$(1, V)
    NEXT R
  NEXT C

  FOR R = 1 TO 100                    'FILL COLUMN 7
    IF V > 675 THEN GOTO TAB7
    W = W + 1
    IF R = 100 THEN W = 3
    IF W = 4 THEN GOTO TAB6
    V = V + 1

    F$(R, 7) = R2$(1, V)
    GOTO TAB7

TAB6:
  W = 0
TAB7:
  NEXT R

  PRINT "SUPER ENCIPHERMENT TABLE COMPLETED"

  ERASE R1$, R2$, Z$                  'MAKE ROOM FOR H$

  OPEN "B:CODE.DAT" FOR RANDOM AS #1 LEN = 1600
  NUM.RECS = LOF(1) / 1600
  PRINT
  PRINT "LAST TABLE ON FILE IS NUMBER  :  "; NUM.RECS
  CLOSE #1
 
  PRINT
  INPUT "ENTER THE NEW TABLE NUMBER    :   ", TABLENUM
 
  PRINT "PRINTING SUPERENCIPHERMENT TABLE "; TABLENUM
TAB16:
  LPRINT TAB(41);
  LPRINT "SUPER ENCIPHERING TABLE No. "; TABLENUM
  LPRINT
  S1$ = "        "
  FOR J1 = 1 TO 2                     'PRINT COLUMN NUMBERS
    LPRINT S1$;
    FOR J = 1 TO 7: LPRINT USING "   ##"; J; : NEXT J
    LPRINT "      ";
  NEXT J1
  LPRINT

  FOR J1 = 1 TO 2                     'PRINT COLUMN HEADING LINES
    LPRINT S1$;
    FOR J = 1 TO 7: LPRINT USING "   \\"; "--"; : NEXT J
    LPRINT "      ";
  NEXT J1
  LPRINT

  FOR R = 1 TO 50
    LPRINT USING "    ### "; R;          'PRINT NUMERIC CODES (LINE NUMBERS)

    FOR C = 1 TO 8
      LPRINT USING "   \\"; F$(R, C); 'PRINT ALPHA EQUIVALENTS
    NEXT C

    LPRINT "  ";
    LPRINT USING "   ### "; R + 50;      'PRINT NUMERIC CODES (LINE NUMBERS)

    FOR C = 1 TO 8
      LPRINT USING "   \\"; F$(R + 50, C);  'PRINT ALPHA EQUIVALENTS
    NEXT C

    LPRINT
  NEXT R

TAB18:
  PRINT
  PRINT "CREATING SUPER DECIPHERMENT TABLE"
  
  FOR R = 1 TO 100
    FOR C = 1 TO 7
      IF F$(R, C) = "  " THEN GOTO TAB8
      X1 = INSTR(C$, MID$(F$(R, C), 1, 1))     ' COL CODE - 1ST ALPHA
      X2 = INSTR(C$, MID$(F$(R, C), 2, 1))     ' COL CODE - 2ND ALPHA
      R$ = STR$(R)
      T$ = "00"
     
      SELECT CASE R
        CASE 1 TO 9
          MID$(T$, 2, 1) = MID$(R$, 2, 1)
        CASE 10 TO 99
          MID$(T$, 1, 2) = MID$(R$, 2, 2)
        CASE 100
          T$ = "00"
      END SELECT

      J$(X1, X2) = T$

TAB8:
    NEXT C
  NEXT R

  PRINT "PRINTING SUPER DICPHERMENT TABLE "; TABLENUM
  PRINT

  LPRINT CHR$(12);                    'FORM FEED TO NEXT PAGE
  S$ = "         "
  LPRINT TAB(33);
  LPRINT "SUPER DECIPHERING TABLE No. "; TABLENUM
  LPRINT

  LPRINT S$;
  FOR X = 1 TO 26: LPRINT USING " \\"; MID$(C$, X, 1); : NEXT X: LPRINT
  LPRINT S$;
  FOR X = 1 TO 26: LPRINT USING " \\"; "--"; : NEXT X: LPRINT

  FOR R = 1 TO 26
    LPRINT "       ";
    LPRINT USING "\\ "; MID$(C$, R, 1);  ' ROW LETTERS

    FOR C = 1 TO 26
      LPRINT USING "\\ "; J$(R, C);
    NEXT C

    LPRINT USING " \\"; MID$(C$, R, 1)   ' ROW LETTERS
    IF R = 26 THEN
      PRINT
    ELSE
      LPRINT
    END IF
  NEXT R

  LPRINT S$;
  FOR X = 1 TO 26: LPRINT USING " \\"; "--"; : NEXT X: LPRINT
  LPRINT S$;
  FOR X = 1 TO 26: LPRINT USING " \\"; MID$(C$, X, 1); : NEXT X: LPRINT

  LPRINT
  LPRINT S$; "Note: 00 = 100"
  LPRINT CHR$(12);                    'FORM FEED TO NEXT PAGE

   
TAB12: '------------------------------ ESTABLISH RANDOM ACCESS DISK FILE

  TYPE TABLE
     ALPHA AS STRING * 1600
  END TYPE

  DIM ARRAY AS TABLE
                                                      
'----------------------------------------------------------------------
  ON F GOTO TAB9, TAB10   ' BRANCH TO CREATE (9) OR READ (10) DISK FILE
'----------------------------------------------------------------------

TAB9:    '---------------------------- SAVE F$(100,8) TO DISK AS H$

  PRINT "CREATING FILE FOR DISK"

  E = 1                               'CREATE H$ FROM F$(100,8)
  FOR R = 1 TO 100
    FOR C = 1 TO 8
      MID$(H$, E, 2) = F$(R, C)
      E = E + 2
    NEXT C
  NEXT R

  OPEN "B:CODE.DAT" FOR RANDOM AS #1 LEN = 1600
  NUM.RECS = LOF(1) / 1600
  PRINT "LAST TABLE ON FILE IS NUMBER  :  "; NUM.RECS
  PRINT
  INPUT "ENTER NEXT TABLE NUMBER       :   ", TABLENUM
  PRINT
  ARRAY.ALPHA = H$                    'MOVE H$ ONTO DISK
  T1 = TABLENUM
  PUT #1, TABLENUM, ARRAY
  TABLENUM = TABLENUM + 1
  CLOSE #1

  CLS
  LOCATE 10, 25
  COLOR 10, 0
  PRINT "TABLE "; T1; " LOADED ONTO DISK"
  SLEEP 5
  GOTO TAB11


TAB10: '------------------------------ RETRIEVE H$ FROM DISK

  CLS
  LOCATE 10, 26
  PRINT "DO YOU WANT TO SEE :"
  LOCATE 12, 20
  PRINT "  1 - SUPER ENCIPHERMENT TABLES"
  LOCATE 14, 20
  PRINT "  2 - SUPER DECIPHERMENT TABLES"
  LOCATE 16, 20
  PRINT "  3 - RETURN TO MAIN MENUE"
TAB10A: Q1$ = INKEY$: IF Q1$ = "" THEN GOTO TAB10A
  IF Q1$ = "3" THEN GOTO START
  IF Q1$ = "2" GOTO TAB14
  IF Q1$ <> "1" THEN GOTO TAB10

  OPEN "B:CODE.DAT" FOR RANDOM AS #1 LEN = 1600

  NUM.RECS = LOF(1) / 1600

  CLS

  LOCATE 10, 26
  PRINT "TABLES ON DISK =   "; NUM.RECS
  LOCATE 12, 26
  INPUT "ENTER TABLE NUMBER  ", TABLENO
  GET #1, TABLENO, ARRAY
  H$ = ARRAY.ALPHA

  CLS
  X = 1
  FOR R1 = 1 TO 5
    LOCATE 12, 21
    PRINT "SUPERENCIPHERMENT TABLE No."; TABLENO
    PRINT

    FOR R = 1 TO 20                   'DISPLAY H$ IN F$(100,8) FORMAT
      R3 = R3 + 1
      PRINT TAB(20);
      PRINT USING "###"; R3;

      FOR C = 1 TO 8
        PRINT USING "  \\"; MID$(H$, X, 2);
        X = X + 2
      NEXT C
      PRINT
    NEXT R

    PRINT "Press Key"
    INPUT "", O
    CLS
  NEXT R1

  INPUT "DO YOU WANT TO PRINT THIS TABLE (Y OR N) "; Q$
  IF Q$ = "Y" THEN GOTO TAB17

  CLOSE #1

TAB11:

  GOTO START

TAB13:  '------------------------------ CREATE BEGINNING RANDOM ALPHA KEYS

  A1$ = C$
  B1$ = C$
  A$ = "                          "
  B$ = A$

  RANDOMIZE TIMER

TAG1:
  R = RND
  Z1 = CINT(27 * R) - 1
  IF Z1 <= 0 THEN GOTO TAG1
  IF Z1 > 26 THEN GOTO TAG1

TAG2:
  C = RND
  Z2 = CINT(27 * C) - 1
  IF Z2 <= 0 THEN GOTO TAG2
  IF Z2 > 26 THEN GOTO TAG2

  IF A = 26 THEN GOTO TAG4
  IF MID$(A1$, Z1, 1) <> " " THEN
      A = A + 1
      MID$(A$, A, 1) = MID$(A1$, Z1, 1)
      MID$(A1$, Z1, 1) = " "
      END IF

TAG4:
  IF B = 26 THEN GOTO TAG5
  IF MID$(B1$, Z2, 1) <> " " THEN
     B = B + 1
     MID$(B$, B, 1) = MID$(B1$, Z2, 1)
     MID$(B1$, Z2, 1) = " "
     END IF
TAG5:

  IF A = 26 AND B = 26 THEN GOTO TAG6
  GOTO TAG1

TAG6:

  RETURN


TAB14: '------------------------------ INSPECT SUPER DECIPHERMENT TABLE

  CLS

  N$ = "00"

  OPEN "B:CODE.DAT" FOR RANDOM AS #1 LEN = 1600

  NUM.RECS = LOF(1) / 1600

  LOCATE 10, 26
  PRINT "TABLES ON DISK =   "; NUM.RECS
  LOCATE 12, 26
  INPUT "ENTER TABLE NUMBER  ", TABLENO
  GET #1, TABLENO, ARRAY
  H$ = ARRAY.ALPHA

  CLS
  Z = 1
  FOR X = 1 TO 100
    A$ = MID$(H$, Z, 16)
    FOR N = 1 TO 16 STEP 2
      IF MID$(A$, N, 1) = " " THEN GOTO TAG7
      R = INSTR(C$, MID$(A$, N, 1))
      C = INSTR(C$, MID$(A$, N + 1, 1))
      J$ = STR$(X)
      IF J$ < "10" THEN
        MID$(N$, 2, 1) = MID$(J$, 2, 1)
        END IF
       
      SELECT CASE X
        CASE 1 TO 9
          J$(R, C) = N$
        CASE 10 TO 99
          J$(R, C) = LTRIM$(J$)
        CASE 100
          J$(R, C) = "00"
        END SELECT

TAG7:
    NEXT N
    Z = Z + 16
  NEXT X

  PRINT TAB(26);
  PRINT "SUPER DECIPHERING TABLE No. "; TABLENO
  PRINT
  PRINT "  ";
  FOR X = 1 TO 26: PRINT USING " \\"; MID$(C$, X, 1); : NEXT X: PRINT
  PRINT "  ";
  FOR X = 1 TO 26: PRINT USING " \\"; "--"; : NEXT X: PRINT


  FOR R = 1 TO 26
    IF R = 10 THEN
      LOCATE 23, 1
      PRINT "Press Key"
      INPUT "", O
      CLS
      END IF
    IF R = 10 THEN GOSUB TAG8

    IF R = 19 THEN
       LOCATE 23, 1
       PRINT "Press Key"
       INPUT "", O
       CLS
       END IF

    IF R = 19 THEN GOSUB TAG8

    PRINT USING "\\ "; MID$(C$, R, 1);  ' ROW LETTERS

    FOR C = 1 TO 26
      PRINT USING "\ \"; J$(R, C);
    NEXT C

    IF R = 26 THEN
      'PRINT
    ELSE
      PRINT
    END IF
  NEXT R

  PRINT " ";
  FOR X = 1 TO 26: PRINT USING " \\"; "--"; : NEXT X: PRINT
  PRINT "  ";
  FOR X = 1 TO 26: PRINT USING " \\"; MID$(C$, X, 1); : NEXT X: PRINT

  PRINT
  PRINT "Press Key to Restart             Note: 00 = 100"

  SLEEP


  GOTO START


TAG8:
  PRINT TAB(26);
  PRINT "SUPER DECIPHERING TABLE No. "; TABLENO
  PRINT

  PRINT "  ";
  FOR X = 1 TO 26: PRINT USING " \\"; MID$(C$, X, 1); : NEXT X: PRINT
  PRINT "  ";
  FOR X = 1 TO 26: PRINT USING " \\"; "--"; : NEXT X: PRINT

RETURN

TAB15:

END

TAB17:
 TABLENUM = TABLENO

 H1 = 1                               'CREATE 100 X 8 ARRAY
 FOR R = 1 TO 100
   FOR C = 1 TO 8
     F$(R, C) = MID$(H$, H1, 2)
     H1 = H1 + 2
   NEXT C
 NEXT R

 IF Q1$ = "1" THEN GOTO TAB16
 IF Q1$ = "2" THEN GOTO TAB18
 PRINT "BAD SELECTION"
 SLEEP
 GOTO START


