REM CRYPTO34 - REVERSING ROTOR CREATION                          (31 AUG 95)

'              THIS PROGRAM CREATES A 100-POSITION ROTOR WHOSE POSITIONS
'              ARE INTERCONNECTED BY 50 "WIRES" ON ONE FACE OF THE ROTOR.

'              THE PURPOSE IS TO CREATE A "REVERSING" ROTOR FOR USE IN THE
'              ENIGMA95 CIPHER MACHINE (ENIGMA95.BAS).

'              THE CONNECTIONS ARE MADE USING THE "INTERVAL" METHOD OF
'              SPACING CONNECTIONS WHICH RESULTS IN THE FLATEST POSSIBLE
'              POLYCHARACTERISTIC DISTRIBUTION.

'              THE BEST METHOD IS START WITH MODE 1, THEN AFTER ENTERING AS
'              MANY 'TO' VALUES AS POSSIBLE, SKIP THROUGH MODES 2 AND 3
'              (WITHOUT USING THEM) AND THEN USE THE AUTOMATIC TRIAL AND
'              ERROR MODE (KEY IN 200).   JUST SIT BACK AND WATCH THE ACTION.
'              WHEN IT IS FINISHED THE COMPUTER WILL BEEP.  YOU RESUME MANUAL
'              INPUT AT THAT POINT.

'----------------------------------------------------------------------------

L = 100                               'NUMBER OF CONTACT POSITIONS
L1 = L / 5                            'DISPLAY COLUMNS LENGTH

CLS
DIM A(L)                               'INTERVAL CONTROL ARRAY
DIM F(L)                               'PRIMARY 'FROM' POSITIONS ARRAY
DIM I(L)                               'INTERVAL DISPLAY ARRAY
DIM T(L)                               'SECONDARY 'TO' POSITIONS ARRAY
DIM U(L)                               'TO POSITIONS CONTROL ARRAY

DIM A$(L)
DIM F$(L)
DIM I$(L)
DIM T$(L)
DIM U$(L)

  GOSUB TAB12                           'INITIALIZE DATA ARRAYS

  B$ = STRING$(80, " ")                 'TO BLANK ENTRY LINES
  T$ = " F  I  T  U  A  "               'COLUMN HEADINGS

RESTORE DATA1           'SET TO READ DATA FILE FOR AUTOMATIC INPUT
                        '  (SEE TAB100)
                        'CODE 500 = LINKED TO PREVIOUS INPUT
                        'CODE 300 = END OF READ DATA FILE
                        'CODE 400 = BLANK
DATA1:
  DATA 076,065,079,087,092,072,400,094,073,400
  DATA 017,041,091,098,069,054,500,030,042,100
  DATA 066,089,067,025,500,056,036,077,033,500
  DATA 071,037,500,075,096,500,500,064,070,093
  DATA 500,500,058,063,097,078,050,090,084,500
  DATA 061,059,055,500,500,500,085,500,500,081
  DATA 500,080,500,500,500,500,500,095,500,500
  DATA 500,500,500,082,500,500,500,500,500,500
  DATA 500,500,400,500,500,400,500,099,500,500
  DATA 500,500,500,500,500,500,500,500,500,500
  DATA 300

  CLS

START:

  LPRINT CHR$(27); "@";               'INITIALIZE PRINTER
  WIDTH LPRINT 160                    'SET LINE WIDTH TO 160 COLUMNS
  LPRINT CHR$(27); "Q"; CHR$(162);    'SET RIGHT MARGIN TO 162 COLUMNS
  LPRINT CHR$(27); "k"; CHR$(1);      'SANS SERIF FONT
  LPRINT CHR$(27); "M";               'ELITE PRINT
  LPRINT CHR$(27); "E";               'EMPHASIZED (BOLD) PRINTING

  CLS
  LOCATE 4, 14
  COLOR 14, 0                         'YELLOW/BLACK
  PRINT "Welcome to CRYPTO34 - THE REVERSING ROTOR GENERATOR"
  COLOR 7, 0                          'WHITE/BLACK

  COLOR 12, 0                         'RED/BLACK
  LOCATE 20, 5
  PRINT TAB(27); "By  Clarence E. Tyner Jr."
  LOCATE 21, 5
  PRINT TAB(16); "Copyright 1995  - All Rights Reserved Worldwide"
  COLOR 7, 0


  COLOR 7, 0                          'WHITE/BLACK
  LOCATE 10, 17
  PRINT "          SELECT OPERATING OPTION"
  PRINT
  PRINT TAB(21); "  1 - ENTER 'TO' VALUES ONLY"
  PRINT
  PRINT TAB(21); "  2 - READ ROTOR FROM FLOPPY DISK"
  PRINT
  INPUT ; "                      ", M4
  PRINT
  IF M4 > 2 OR M4 < 1 GOTO START

   ON M4 GOTO TAB0, TAB2Y
TAB0: '----------Mode 1--------------- ENTER THE 'TO' VALUE (ROTOR REAR)
TAB0A:

SKIP9:

  IF Q = 1 OR Q = 2 GOTO SWITCH
  CLS
  LOCATE 10, 24
  PRINT "SELECT INPUT METHOD"
  LOCATE 12, 22
  PRINT "  1 - AUTOMATIC FROM DATA STATEMENTS  (DEMO)"
  LOCATE 14, 22
  PRINT "  2 - MANUAL THRU KEYBOARD"
  LOCATE 16, 24
  INPUT "", Q
  IF Q > 2 GOTO SKIP9


SWITCH:

  GOSUB TAB12                         'INITIALIZE DATA ARRAYS

  ON Q GOTO INPUT1, INPUT2

INPUT1:                               'AUTOMATIC INPUT FROM DATA STATEMENTS
  LOCATE 24, 1
  Z = Z + 1                           'INCREMENT POSITION NUMBER
  READ V
  IF V = 300 THEN                     'END OF FILE
     GOTO TAB8                        'GOTO MODE 2A
     END IF
  IF V = 500 GOTO INPUT1              'LINKED INPUT
  IF V = 400 GOTO INPUT1              'BLANK DATA STATEMENT
  IF V = 9999 GOTO TAB8               'TO MODE 2A
  

INPUT2:                               'MANUAL KEYBOARD INPUT

TAB1: '------------------------------- START PROGRAM / CONTINUE MODE 1

  GOSUB TAB4A
  LOCATE (L / 5) + 3, 1
  PRINT B$;
  LOCATE (L / 5) + 4, 1
  PRINT B$;

TAB2:  '----------------------------- 'MODE 1 INPUT

  IF Q = 1 GOTO PASS5                 'AUTOMATIC INPUT

  Z = Z + 1                           'ADVANCE TO NEXT POSITION

PASS5:

  IF Z > L THEN Z = Z - 1
  IF I(Z) + T(Z) <> 400 THEN          'CHECK FOR UNUSED POSITIONS
    GOTO TAB2                         'SELECT NEXT LINE
  END IF

IF Q = 1 THEN GOTO TEST1

TAB3: '------------------------------ 'MODE 1 INPUT

  LOCATE (L / 5) + 3, 1
  PRINT "MODE 1 : FROM"; Z; " ";
  INPUT "ENTER A 'TO' VALUE (OR '9999' FOR MODE 2) "; V
  IF V < 1 THEN                       'BAD INPUT - RE-DO
     BEEP
     GOTO TAB3
     END IF
  LOCATE (L / 5) + 3, 1
  PRINT B$

TEST1:
  IF V = 9999 THEN GOTO TAB8          'TO MODE 2A
  IF V > L THEN                       'BAD INPUT - RE-DO
    BEEP
    GOTO TAB3
    END IF
  T(Z) = V
  I5 = T(Z) - Z                       '1ST INTERVAL

  FOR J = 1 TO L                      '1ST DUPLICATE INTERVAL CHECK
    IF I(J) <> I5 THEN
      GOTO SKIP
    ELSE
      BEEP                            'DUPLICATE FOUND
      T(Z) = 200
      GOTO TAB3                       'SELECT ANOTHER 'TO' VALUE
    END IF
SKIP:
    NEXT J
  I(Z) = I5

  F = T(Z)
  I = L - I(Z)                        '2ND INTERVAL

  FOR J = 1 TO L                      '2ND DUPLICATE INTERVAL CHECK
    IF I <> I(J) OR I = (L / 2) THEN
      GOTO SKIP2
    ELSE
      BEEP                            'DUPLICATE FOUND
      T(Z) = 200
      GOTO TAB3                       'SELECT ANOTHER 'TO' VALUE
SKIP4:
    END IF
SKIP2:
  NEXT J

  T = F(Z)

  IF I(Z) < 0 THEN
    I(Z) = I(Z) + L
    END IF

  IF I(T(Z)) + T(T(Z)) <> 400 OR I(Z) = 200 THEN
    T(Z) = 200
    I(Z) = 200
    Z = Z - 1
    BEEP
    GOTO TAB1
  ELSE
    F(T(Z)) = F
    I(T(Z)) = I
    T(T(Z)) = T
    U(T(Z)) = T(Z)
    U(T) = T
    A(I(Z)) = I(Z)
    A(I) = I
    END IF

  FOR X = 1 TO L                      'CONVERT NUMERIC TO ALPHA
    I$(X) = STR$(I(X)): IF I$(X) = " 200" THEN I$(X) = " "
    T$(X) = STR$(T(X)): IF T$(X) = " 200" THEN T$(X) = " "
    U$(X) = STR$(U(X)): IF U$(X) = " 200" THEN U$(X) = " "
    A$(X) = STR$(A(X)): IF A$(X) = " 200" THEN A$(X) = " "
  NEXT X
 
  GOSUB TAB4A                         'DISPLAY RESULTS

  IF Q = 1 THEN GOTO INPUT1           'REPEAT AUTOMATIC INPUT

GOTO TAB2  '------------------------- 'REPEAT MODE 1 INPUT


TAB8: '------------------------------ 'MODE 2A

  LOCATE L1 + 3, 1
  'PRINT "MODE 2 : LAST = "; Z; " ";
  PRINT "MODE 2 :   ";
  INPUT "ENTER  9999  FOR MODE 3    ", Z
  LOCATE L1 + 3, 1
  PRINT B$
  Q = 2                               'MANUAL INPUT MODE
  IF Z = 9999 GOTO TAB5               'TO MODE 3
  GOTO TAB3


TAB5: '------------------------------ 'MODE 3

  LOCATE L1 + 3, 1
  PRINT B$
  LOCATE L1 + 3, 1
  PRINT "MODE 3 : ";
  INPUT "  ENTER 200 FOR AUTOMATIC MODE   ", P
  IF P = 200 GOTO TAB100              'AUTOMATIC MODE 4
  IF T(P) = 200 THEN                  'CHECK FOR ALREADY BLANK POSITION
    BEEP
    GOTO TAB5                         'RE-ENTER SELECTION
    END IF
  LOCATE L1 + 3, 1
  PRINT B$

  W1 = T(P)                           'TO HOLD BLANKED VALUES
  W2 = T(W1)
 
  A(I(P)) = 200: A$(I(P)) = " "
  A(I(T(P))) = 200: A$(I(T(P))) = " "
  U((T(P))) = 200: U$((T(P))) = " "
  T(T(P)) = 200: T$(T(P)) = ""
  U(P) = 200: U$(P) = " "
  I(T(P)) = 200: I$(T(P)) = " "
  T(P) = 200: T$(P) = ""
  I(P) = 200: I$(P) = " "

  FOR X = 1 TO L                      'CONVERT NUMERIC TO ALPHA
    I$(X) = STR$(I(X)): IF I$(X) = " 0" THEN I$(X) = " "
    T$(X) = STR$(T(X)): IF T$(X) = " 0" THEN T$(X) = " "
    U$(X) = STR$(U(X)): IF U$(X) = " 0" THEN U$(X) = " "
    A$(X) = STR$(A(X)): IF A$(X) = " 0" THEN A$(X) = " "
  NEXT X
    
  GOSUB TAB4A                         'DISPLAY CURRENT RESULTS
  Z = 0

TAB4: '------------------------------- MODE 3 INPUT

  LOCATE L1 + 3, 1
  PRINT B$;
  LOCATE L1 + 4, 1
  PRINT B$;
  LOCATE L1 + 3, 1
  PRINT "MODE 3 : ";
  INPUT "ENTER A BLANK 'FROM' POSITION  (OR '9999' TO BLANK ANOTHER) "; Z
  
  IF Z = 9999 GOTO TAB5               'BLANK ANOTHER POSITION
  IF Z > L THEN Z = Z - 1             'ADJUST TOO LARGE INPUT
  IF I(Z) + T(Z) <> 400 THEN          'CHECK FOR UNUSED POSITIONS
    BEEP
    GOTO TAB4
    END IF

 
TAB3A: '----------------------------- 'MODE 4 - TRY ALTERNATE FROM/TO

  IF V = Z THEN GOTO SKIP10
  LOCATE L1 + 3, 1
  PRINT B$
  LOCATE L1 + 3, 1
  COLOR 12, 0                         'RED/BLACK
  PRINT "MODE 4 : FROM "; Z; ":";
  COLOR 14, 0                         'YELLOW/BLACK
  PRINT " ENTER A 'TO' VALUE  ";
  COLOR 12, 0                         'RED/BLACK
  PRINT " OR '"; Z; "'FOR A NEW 'FROM' POSITION"
  LOCATE L1 + 4, 1
  COLOR 10, 0                         'GREEN/BLACK
  PRINT "                   OR '9999' TO BLANK ANOTHER POSITION ";
  INPUT ; "", V
SKIP10:
  
  LOCATE L1 + 4, 1
  PRINT B$;

  GOSUB TAB4A                         'DISPLAY RESULTS

  IF V = Z THEN
    BEEP
    GOTO TAB4                         'TO MODE 3 INPUT FOR A NEW POSITION
    END IF
  IF V = 9999 GOTO TAB5               'TO MODE 3 TO BLANK ANOTHER POSITION
  T(Z) = V
  IF Z < T(Z) THEN
     I5 = T(Z) - Z
  ELSE
     I5 = L - ABS(T(Z) - Z)           '1ST INTERVAL
  END IF

REM TEST 1
  FOR J = 1 TO L                      '1ST DUPLICATE INTERVAL CHECK
    IF I(J) <> I5 THEN
       GOTO SKIPA
    ELSE
       BEEP
       T(Z) = 200
       GOTO TAB3A                     'DUPLICATE FOUND - GO TO MODE 4 INPUT
    END IF

SKIPA:
  NEXT J
  I(Z) = I5

  F = T(Z)
  I = L - ABS(I(Z))                   '2ND INTERVAL
  
REM TEST 2
  FOR J = 1 TO L                      '2ND DUPLICATE INTERVAL CHECK
    IF I(J) <> I THEN
      GOTO SKIP2A
    ELSE
      BEEP
      GOTO TAB3A                      'DUPLICATE FOUND - GO TO MODE 2 INPUT
    END IF
SKIP2A:
  NEXT J

  T = F(Z)
  IF I(Z) < 0 THEN                    'TEST FOR NEGATIVE
     I(Z) = I(Z) + L                  'PRODUCE POSITIVE INTERVAL
     END IF

  IF I(T(Z)) + T(T(Z)) <> 400 OR I(Z) = 200 THEN  'TEST FOR BLANK CELLS
     T(Z) = 200
     I(Z) = 200
     Z = Z - 1
     BEEP                             'CELLS NOT BLANK
     GOTO TAB1                        'GO TO MODE 1
  ELSE                                'FILL CELLS
     F(T(Z)) = F
     I(T(Z)) = I
     T(T(Z)) = T
     U(T(Z)) = T(Z)
     U(T) = T
     A(I(Z)) = I(Z)
     A(I) = I
   END IF

  FOR X = 1 TO L                      'CONVERT TO ALPHA FORMAT
     I$(X) = STR$(I(X)): IF I$(X) = " 0" THEN I$(X) = " "
     T$(X) = STR$(T(X)): IF T$(X) = " 0" THEN T$(X) = " "
     U$(X) = STR$(U(X)): IF U$(X) = " 0" THEN U$(X) = " "
     A$(X) = STR$(A(X)): IF A$(X) = " 0" THEN A$(X) = " "
  NEXT X

  GOSUB TAB4A                         'DISPLAY RESULTS

  V = Z

  GOTO TAB3A                          'GO TO MODE 2

TAB4A:   '---------------------------- DISPLAY CURRENT RESULTS

  LOCATE 1, 1
  COLOR 7, 0                          'WHITE/BLACK

  LOCATE 2, 1
  FOR Y = 1 TO 5                      'HEADINGS
    PRINT T$;
  NEXT Y
  PRINT

  N1$ = "## "
  N2$ = "##  "
  N3$ = "\\ "
  N4$ = "\\  "

  LOCATE 3, 1
  FOR R = 1 TO 20
    K = -20

    FOR C1 = 1 TO 5
      COLOR 12, 0                     'RED / BLACK - FROM COLUMN
      K = K + 20
      IF F(R + K) = L THEN
        PRINT USING N3$; "00";
        GOTO SKIP0
        END IF
      PRINT USING N1$; F(R + K);
SKIP0:
      COLOR 7, 0                      'WHITE/BLACK - INTERVAL COLUMN
      IF I(R + K) = L THEN
        PRINT USING N3$; "00";
        GOTO SKIP21
        END IF
  
      IF I(R + K) = 200 THEN GOTO SKIP1
      PRINT USING N3$; RIGHT$(I$(R + K), 2);
      GOTO SKIP21

SKIP1:
      PRINT USING N3$; "";

SKIP21:
     IF T(R + K) = L THEN             'TO COLUMN
       PRINT USING N3$; "00";
       GOTO SKIP41
       END IF

    IF T(R + K) = 200 THEN GOTO SKIP31
      PRINT USING N3$; RIGHT$(T$(R + K), 2);
      GOTO SKIP41
  
SKIP31:
    IF (R + K) = W1 GOTO SKIP50
    IF (R + K) = W2 GOTO SKIP60
    PRINT USING N3$; "";
    GOTO SKIP41
SKIP50:
    COLOR 11, 0                       'LT CYAN/BLACK
    PRINT USING N3$; "00";
    COLOR 7, 0                        'WHITE/BLACK
    GOTO SKIP41
SKIP60:
    COLOR 10, 0                       'LT GREEN/BLACK
    PRINT USING N3$; "00";
    COLOR 7, 0                        'WHITE / BLACK
 
SKIP41:
    IF U(R + K) = 200 THEN GOTO SKIP5
    COLOR 14, 0                       'YELLOW/BLACK - U COLUMN
    IF U(R + K) = L THEN
      PRINT USING N3$; "00";
      GOTO SKIP6
      END IF
    
      PRINT USING N3$; RIGHT$(U$(R + K), 2);
      COLOR 7, 0                      'WHITE/BLACK
      GOTO SKIP6

SKIP5:
      PRINT USING N3$; "";

SKIP6:
      IF A(R + K) = L THEN            'A COLUMN
        PRINT USING N4$; "00";
        GOTO SKIP8
        END IF

      IF A(R + K) = 200 THEN GOTO SKIP7
      PRINT USING N4$; RIGHT$(A$(R + K), 2);
      GOTO SKIP8

SKIP7:
      PRINT USING N4$; "";

SKIP8:
    NEXT C1
    PRINT
  NEXT R

  J = 1                               'FLAG TO PREVENT DISPLAY REPEATS

  B5 = 0                              'TEST FOR COMPLETION
  FOR B = 1 TO L
    IF T(B) <> 200 THEN               'COUNT FILLED CELLS
      B5 = B5 + 1
      END IF
    IF B5 = 98 THEN                   'QUIT CONDITION SATISIFIED
      GOTO TAB120                     'DETERMINE RUN TIME
      END IF
  NEXT B

RETURN



TAB100: '----------------------------- AUTOMATIC MODE 4

  T1 = TIMER                          'START OF TIME

TAB100A:
  Z = 0: V = 0: I5 = 0: I = 0
  C = C + 1                           '1ST CYCLE COUNT

  FOR F = 1 TO L                      'SELECT A POSITION TO BLANK
  
    P = F
    IF T(P) = 200 GOTO SKIP11         'POSITION ALREADY BLANKED

    W1 = T(P)                         'TO HOLD BLANKED 'TO' VALUES
    W2 = T(W1)
 
    A(I(P)) = 200: A$(I(P)) = " "     'BLANK POSITIONS
    A(I(T(P))) = 200: A$(I(T(P))) = " "
    U((T(P))) = 200: U$((T(P))) = " "
    T(T(P)) = 200: T$(T(P)) = ""
    U(P) = 200: U$(P) = " "
    I(T(P)) = 200: I$(T(P)) = " "
    T(P) = 200: T$(P) = ""
    I(P) = 200: I$(P) = " "

    FOR X = 1 TO L                    'CONVERT NUMERIC TO ALPHA
      I$(X) = STR$(I(X)): IF I$(X) = " 0" THEN I$(X) = " "
      T$(X) = STR$(T(X)): IF T$(X) = " 0" THEN T$(X) = " "
      U$(X) = STR$(U(X)): IF U$(X) = " 0" THEN U$(X) = " "
      A$(X) = STR$(A(X)): IF A$(X) = " 0" THEN A$(X) = " "
    NEXT X
   
    GOSUB TAB4A                       'DISPLAY RESULTS
 
    FOR Z = 1 TO L                    'SELECT A 'FROM' POSITION TO TRY
    
      IF I(Z) + T(Z) <> 400 GOTO SKIP12 'TEST 'FROM' POSITION FOR BLANKS


      FOR V = 1 TO L                  'SELECT A 'TO' POSITION TO TRY
        I5 = 0: I = 0                 'INITIALIZE INTERVAL VALUES
        
        IF V = Z GOTO SKIP13          'TO AVOID ZERO INTERVAL
        IF T(V) <> 200 GOTO SKIP13    'TEST FOR BLANK CELL
       
        IF (Z = W1 OR Z = W2) AND (V = W1 OR V = W2) GOTO SKIP13

        T(Z) = V
        IF Z < T(Z) THEN
          I5 = T(Z) - Z
        ELSE
          I5 = L - ABS(T(Z) - Z)      '1ST INTERVAL
          END IF

        FLAG = 50
        GOSUB TAB110
        'SLEEP
  
    REM TEST 1
        FOR J = 1 TO L                '1ST DUPLICATE INTERVAL CHECK
          IF I(J) <> I5 THEN
            GOTO SKIPA1
          ELSE
            T(Z) = 200
            GOTO SKIP13               'DUPLICATE FOUND - GO TO MODE 2 INPUT
            END IF

SKIPA1:
        NEXT J
    
        I = L - I5                    '2ND INTERVAL
    
        GOSUB TAB110
    
    REM TEST 2
    FOR J = 1 TO L                    '2ND DUPLICATE INTERVAL CHECK
      IF I(J) <> I THEN
        GOTO SKIP2A1
      ELSE
        GOTO SKIP13
        END IF
SKIP2A1:
    NEXT J

    I(Z) = I5
    T = F(Z)
    IF I(Z) < 0 THEN                  'TEST FOR NEGATIVE
      I(Z) = I(Z) + L                 'PRODUCE POSITIVE INTERVAL
      END IF

    IF I(T(Z)) + T(T(Z)) <> 400 OR I(Z) = 200 THEN  'TEST FOR BLANK CELLS
      T(Z) = 200
      I(Z) = 200
      Z = Z - 1
      GOTO SKIP12                     'CELLS NOT BLANK - GO TO MODE 1 INPUT
    ELSE                              'CELLS BLANK - FILL CELLS
      I(T(Z)) = I
      T(T(Z)) = T
      U(T(Z)) = T(Z)
      U(T) = T
      A(I(Z)) = I(Z)
      A(I) = I
      END IF

    FOR X = 1 TO L                    'CONVERT TO ALPHA FORMAT
      I$(X) = STR$(I(X)): IF I$(X) = " 0" THEN I$(X) = " "
      T$(X) = STR$(T(X)): IF T$(X) = " 0" THEN T$(X) = " "
      U$(X) = STR$(U(X)): IF U$(X) = " 0" THEN U$(X) = " "
      A$(X) = STR$(A(X)): IF A$(X) = " 0" THEN A$(X) = " "
    NEXT X

    GOSUB TAB110
  
    GOSUB TAB4A                       'DISPLAY RESULTS
    EXIT FOR
SKIP13:
  NEXT V
  V = 0
SKIP12:
  NEXT Z
  Z = 0

SKIP11:
  NEXT F
  F = 0
  GOTO TAB100A                        'RECYCLE

END

TAB110: '----------------------------- STATUS LINE
  H = H + 1
  LOCATE 23, 1
  PRINT B$;
  LOCATE 24, 1
  PRINT USING "CYCLE ###  BLANK =###  FROM =###  TO =###  POSITIONS FILLED = ##  TRY =#######"; C; F; Z; V; B5; H;
 
  RETURN


TAB120:  '---------------------------- DETERMINATION OF TIME OF RUN

  GOSUB TAB90                         'DETERMINE VACANT POSITIONS

  LOCATE 23, 1
  PRINT B$;                           'BLANK THE LINE
  LOCATE 24, 1
  PRINT B$;                           'BLANK THE LINE
  LOCATE 23, 1
  PRINT USING "CYCLE ###   TRY ######   ## POSITIONS FILLED     BLANK POSITIONS : ### AND ###"; C; H; B5; J7; J9
  T3 = TIMER                          'END TIME
  T5 = (T3 - T1) / 3600               'HOURS
  T6 = (T5 - FIX(T5)) * 60            'MINUTES
  T7 = (T6 - FIX(T6)) * 60            'SECONDS
  LOCATE 24, 1
  PRINT "Elapsed Time : ";
  PRINT USING "## hours ## minutes ## seconds"; FIX(T5); FIX(T6); CINT(T7);
  BEEP
  INPUT "  ENTER 9999 TO PRINT ROTOR  ", O
  IF O = 9999 GOTO TAB60

  GOTO START


TAB90: '------------------------------ LOCATE EMPTY ARRAY CELLS

   FOR J7 = 1 TO L                    'LOCATE 1ST UNUSED 'TO' POSITION
     IF U(J7) = 200 THEN
     EXIT FOR
     END IF
   NEXT J7

 
   FOR J8 = 1 TO L                    'LOCATE UNUSED INTERVAL
     IF A(J8) = 200 THEN
     EXIT FOR
     END IF
   NEXT J8

   FOR J9 = J7 + 1 TO L               'LOCATE 2ND UNUSED 'TO' POSITION
     IF T(J9) = 200 THEN
     EXIT FOR
     END IF
   NEXT J9

RETURN


TABX:   '----------------------------- ROUTINE TO MAKE ROTOR DISK FILES

  CLS
  PRINT "DO YOU WANT TO : "
  PRINT
  PRINT "  1 - CREATE A ROTOR FILE ON THE DISK"
  PRINT
  PRINT "  2 - READ A ROTOR FILE FROM THE DISK"
  PRINT
  INPUT Q7
  ON Q7 GOTO TAB1Y, TAB2Y
  GOTO TABX


TAB1Y:

  GOSUB TAB90                         'DETERMINE VACANT POSITIONS

  CLS
  LOCATE 10, 10
  PRINT "1ST VACANT POSITION IS "; J7
  LOCATE 12, 10
  PRINT "2ND VACANT POSITION IS "; J9
  LOCATE 14, 10

  T(J7) = J9
  T(J9) = J7

  PRINT "THESE CONNECTIONS HAVE BEEN WIRED ON THE ROTOR"
 
  LOCATE 16, 10
  INPUT ; "INSERT ROTOR DISK INTO DRIVE B THEN PRESS 'ENTER'", Q
  CLS

  R$ = STRING$(200, "0")

  Y = 1
  FOR B = 1 TO 100
     MID$(R$, Y, 2) = RIGHT$(STR$(T(B)), 2)
     Y = Y + 2
  NEXT B
     

  PRINT
  PRINT "ROTOR CREATED"
  PRINT

'--------------------------------------------------------
REM - ROUTINE TO CREATE RANDOM-ACCESS DISK FILE OF ROTORS
'--------------------------------------------------------

  TYPE REVROTRSET
       REVROTR AS STRING * 200
  END TYPE

  DIM REVROTRS AS REVROTRSET

  OPEN "B:REVROTRS.DAT" FOR RANDOM AS #1 LEN = 200

  NUM.RECS = LOF(1) / 200
  PRINT "LAST ROTOR ON FILE IS NUMBER : "; NUM.RECS
  PRINT
  INPUT "ENTER NEXT ROTOR NUMBER      :  ", REVROTRNO
  PRINT

  REVROTRS.REVROTR = R$
  PUT #1, REVROTRNO, REVROTRS
  REVROTRNO = REVROTRNO + 1

  CLOSE #1

  PRINT "1 REVERSING ROTOR DISK ADDED"
  PRINT



TAB2Y:  '-----------------------ROUTINE TO READ ROTORS FROM FILE
  CLS
  OPEN "B:REVROTRS.DAT" FOR RANDOM AS #1 LEN = 200

  NUM.RECS = LOF(1) / 200

  DO
    CLS
    PRINT
    INPUT ; "ENTER ROTOR NUMBER TO SEE CONTENTS :  ", REVROTRNUM
    CLS
    LOCATE 1, 1
    PRINT TAB(30); "REVERSING ROTOR No. "; REVROTRNUM
'   IF ((REVROTRNUM <= NUM.RECS) AND (REVROTRNUM > 0)) THEN GET #1, REVROTRNUM, REVROTRS
    PRINT
    GET #1, REVROTRNUM, REVROTRS
    PRINT

    Y3 = 0
    U5 = 1
    FOR X9 = 1 TO 4
      PRINT USING "\   \"; "From ";
      FOR Y7 = 1 TO 25
        IF Y3 + Y7 = 100 THEN
           PRINT USING "\ \"; " 00";
        ELSE
           PRINT USING "###"; Y3 + Y7;
        END IF
      NEXT Y7
      PRINT
      Y3 = Y3 + 25

      GOSUB TABZ                      'DISPLAY DASHED LINE

      PRINT USING "\   \"; "To   ";
      FOR U = 1 TO 25
        PRINT USING " \\"; MID$(REVROTRS.REVROTR, U5, 2);
        U5 = U5 + 2
      NEXT U
    PRINT : PRINT : PRINT
    NEXT X9

    INPUT "DO YOU WANT TO SEE ANOTHER ROTOR (Y or N)  ", ANS$
  LOOP UNTIL (ANS$ = "N")
 
  GOTO TAB13                          'END PROGRAM

TABZ: '------------------------------- DISPLAY DASHED LINE

  PRINT USING "\   \"; "     ";
  COLOR 14, 0                         'YELLOW/BLACK
  FOR Z = 1 TO 25
  PRINT USING " \\"; "--";
  NEXT Z
  COLOR 7, 0                          'WHITE/BLACK
  PRINT
  RETURN



TAB60:  '--------Mode 4--------------- PRINT ROTOR

  CLS
  LOCATE 12, 20

  INPUT "ENTER ROTOR NUMBER : ", R7$

  LPRINT
  LPRINT CHR$(27); "W"; CHR$(1);      'DOUBLE WIDE

  LPRINT "Rotor "; R7$
  LPRINT

  LPRINT CHR$(27); "W"; CHR$(0);      'DOUBLE WIDE CANCEL

  E = 25
  D = -E
  FOR Y = 1 TO 4
    D = D + E

    LPRINT USING "\\:"; "F ";
    FOR X = 1 TO E
      IF F(X + D) = L THEN
        LPRINT USING " \\"; "00";
        GOTO PASS1
        END IF
      LPRINT USING " ##"; F(X + D);
PASS1:
    NEXT X
    LPRINT

    LPRINT USING "\\:"; "I ";
    FOR X = 1 TO E
      IF I(X + D) = 200 THEN
        LPRINT USING " \\"; "";
        GOTO PASS2
      END IF
      LPRINT USING " ##"; I(X + D);
PASS2:
    NEXT X
    LPRINT : 'LPRINT

    LPRINT USING "\\:"; "T ";
    FOR X = 1 TO E

      IF T(X + D) = 200 THEN
         LPRINT USING " \\"; "";
         GOTO PASS3
      END IF

      IF T(X + D) = L THEN
        LPRINT USING " \\"; "00";
        GOTO PASS3
      END IF

      LPRINT USING " ##"; T(X + D);
PASS3:
    NEXT X
    LPRINT

    FOR X = 1 TO E + 1
      LPRINT USING "\ \"; "---";
    NEXT X

    LPRINT
    LPRINT USING "\\:"; "U ";
    FOR X = 1 TO E

      IF U(X + D) = 200 THEN
        LPRINT USING " \\"; "";
        GOTO PASS4
      END IF

      IF U(X + D) = L THEN
        LPRINT USING " \\"; "00";
        GOTO PASS4
      END IF

      LPRINT USING " ##"; U(X + D);

PASS4:
    NEXT X
    LPRINT

    LPRINT USING "\\:"; "A ";
    FOR X = 1 TO E
      IF A(X + D) = 200 THEN
        LPRINT USING " \\"; "";
        GOTO PASS5A
      END IF
      LPRINT USING " ##"; A(X + D);

PASS5A:
    NEXT X
    LPRINT : LPRINT : LPRINT : LPRINT
     
  NEXT Y
  LPRINT : LPRINT
  'GOTO START
  GOTO TAB1Y                          'TO CREATE A ROTOR IN REVROTR.DAT

TAB12: '-------------------------------INITIALIZE DATA ARRAYS

FOR Y = 1 TO L
  F(Y) = Y
  I(Y) = 200
  T(Y) = 200
  U(Y) = 200
  A(Y) = 200
NEXT Y

RETURN

TAB13:
 
 END

